from __future__ import unicode_literals, absolute_import

import logging

import fedora_messaging

from .tag import TagSigner
from .atomic import AtomicSigner
from .coreos import CoreOSSigner


log = logging.getLogger('robosignatory')


class Consumer(object):
    """All messages are received by this class's __call__() method."""

    def __init__(self):
        log.info('Initializing Robosignatory consumer')
        self.config = fedora_messaging.config.conf["consumer_config"]
        try:
            self.tag_handler = TagSigner(self.config)
        except:
            log.exception("Error initializing Tag handler, ignoring")
            self.tag_handler = None
        try:
            self.atomic_handler = AtomicSigner(self.config)
        except:
            log.exception("Error initializing Atomic handler, ignoring")
            self.atomic_handler = None
        try:
            self.coreos_handler = CoreOSSigner(self.config)
        except:
            log.exception("Error initializing CoreOS handler, ignoring")
            self.coreos_handler = None

    def __call__(self, msg):
        """
        Callback method called by fedora-messaging consume.

        Redirect messages to the correct handler using the
        message topic.

        In case of duplicate messages, robosignatory will just try to re-sign
        the object, which will have no effect (except wasted cyles but that
        should not be a major problem).

        Args:
            msg (fedora_messaging.api.Message): The message received from the broker.
        """
        log.info('Received message from fedora-messaging with topic: %s', msg.topic)

        try:
            if msg.topic.endswith('.buildsys.tag'):
                if self.tag_handler:
                    log.debug('Passing message to the Tag handler')
                    self.tag_handler.consume(msg)
                else:
                    log.debug('Received message for disabled Tag handler')
            elif msg.topic.endswith('.pungi.compose.ostree'):
                if self.atomic_handler:
                    log.debug('Passing message to the Atomic handler')
                    self.atomic_handler.consume(msg)
                else:
                    log.debug('Received message for disabled Atomic handler')
            elif (msg.topic.endswith('.coreos.build.request.artifacts-sign')
                  or msg.topic.endswith('.coreos.build.request.ostree-sign')):
                if self.coreos_handler:
                    log.debug('Passing message to the CoreOS handler')
                    self.coreos_handler.consume(msg)
                else:
                    log.debug('Received message for disabled CoreOS handler')
        except Exception as e:
            error_msg = '{e}: Unable to handle message: {msg}'.format(e=e, msg=msg)
            log.exception(error_msg)
            raise fedora_messaging.exceptions.Nack(error_msg)
